==> apache-netbeans-java-20.0.301-sources.zip: ide/languages.hcl/src/org/netbeans/modules/languages/hcl/ast/HCLCollection.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.languages.hcl.ast;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringJoiner;

/**
 *
 * @author lkishalmi
 */
public abstract class HCLCollection<T> extends HCLExpression {

    public final List<T> elements;

    public HCLCollection(List<T> elements) {
        this.elements = Collections.unmodifiableList(elements);
    }
    
    public static final class Tuple extends HCLCollection<HCLExpression> {


        public Tuple(List<HCLExpression> elements) {
            super(elements);
        }
        
        @Override
        public String asString() {
            StringJoiner sj = new StringJoiner(",", "[", "]");
            for (HCLExpression element : elements) {
                sj.add(element.asString());
            }
            return sj.toString();
        }

        @Override
        public List<? extends HCLExpression> getChildren() {
            return elements;
        }
        
    }
    
    public static final class ObjectElement {
        public final HCLExpression key;
        public final HCLExpression value;
        public final int group;

        public ObjectElement(HCLExpression key, HCLExpression value, int group) {
            this.key = key;
            this.value = value;
            this.group = group;
        }

        @Override
        public String toString() {
            return key.asString() + "=" + value.asString();
        }
        
        
    }
    
    public static final class Object extends HCLCollection<ObjectElement> {

        private final List<? extends HCLExpression> parts;
        private final List<? extends HCLExpression> keys;
        private final List<? extends HCLExpression> values;
        
        public Object(List<ObjectElement> elements) {
            super(elements);
            List<HCLExpression> p = new ArrayList<>(elements.size() * 2);
            List<HCLExpression> k = new ArrayList<>(elements.size());
            List<HCLExpression> v = new ArrayList<>(elements.size());
            for (ObjectElement e : elements) {
                k.add(e.key);
                v.add(e.value);
                p.add(e.key);
                p.add(e.value);
            }
            parts = Collections.unmodifiableList(p);
            keys = Collections.unmodifiableList(k);
            values = Collections.unmodifiableList(v);
        }

        @Override
        public String asString() {
            StringJoiner sj = new StringJoiner(",", "{", "}");
            for (ObjectElement element : elements) {
                sj.add(element.toString());
            }
            return sj.toString();
        }

        public List<? extends HCLExpression> getKeys() {
            return keys;
        }
        
        public List<? extends HCLExpression> getValues() {
            return values;
        }
        
        @Override
        public List<? extends HCLExpression> getChildren() {
            return parts;
        }
        
    }
}

==> apache-netbeans-java-21.9.9-sources.zip: ide/languages.hcl/src/org/netbeans/modules/languages/hcl/ast/HCLCollection.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.languages.hcl.ast;

import java.util.List;
import java.util.StringJoiner;

/**
 *
 * @author lkishalmi
 */
public sealed interface HCLCollection<T> extends HCLExpression {

    public record Tuple(List<HCLExpression> elements) implements  HCLCollection<HCLExpression> {

        public Tuple {
            elements = List.copyOf(elements);
        }

        @Override
        public String asString() {
            StringJoiner sj = new StringJoiner(",", "[", "]");
            for (HCLExpression element : elements) {
                sj.add(element.asString());
            }
            return sj.toString();
        }

    }

    public record ObjectElement(HCLExpression key, HCLExpression value) implements HCLExpression {
        @Override
        public String asString() {
            return key.asString() + "=" + value.asString();
        }

        @Override
        public List<? extends HCLExpression> elements() {
            return List.of(key, value);
        }
    }
    
    public record Object(List<ObjectElement> elements) implements HCLCollection<ObjectElement> {

        public Object {
            elements = List.copyOf(elements);
        }

        @Override
        public String asString() {
            StringJoiner sj = new StringJoiner(",", "{", "}");
            for (ObjectElement element : elements) {
                sj.add(element.toString());
            }
            return sj.toString();
        }        
    }
}
